class ComicStory < Peta::Leaf
  load_manifest
  belongs_to :author
  belongs_to :comic
  belongs_to :story
  
  validates :comic_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :story_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  scope :with_comic, -> do
    includes(:comic)
  end
  
  scope :find_index, -> do
    with_comic.where(Comic.arel_table[:visible].gt 0).references(:comic)
  end
  
  scope :find_private, -> (operators) do 
    with_comic.where(Comic.arel_table[:author_id].eq operators.author.id).references(:comic)
  end
  
  scope :by_comic, -> (comic_id) do 
    where(comic_id: comic_id)
  end
  
  scope :find_by_comic, -> (comic_id) do 
    find_index.by_comic(comic_id)
  end
  
  scope :find_by_story, -> (story_id) do 
    find_index.where(story_id: story_id).references(:comic)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(Comic.arel_table[:author_id].eq author_id).references(:comic)
  end
  
  scope :find_play, -> (comic_id) do 
    find_by_comic(comic_id)
  end
  
  scope :find_private_play, -> (comic_id, operators) do 
   find_private(operators).by_comic(comic_id)
  end
  
  def supply_default
    self.comic_id = nil
    self.story_id = nil
    self.t = nil
  end
  
  def overwrite operators
  end
  
  def disp_t
    self.t + 1
  end
  
  def disp_t_by_text
    I18n.t('comic_stories.show.t', :t => self.disp_t)
  end
  
  def title
    self.disp_t_by_text + ':' + self.story.title
  end
  
  def self.public_list_where list
    'comics.visible > 0'
  end
  
  def self.by_author_list_includes
    {
      :comic => {
        :author => {}
      }
    }
  end
  
  def self.show_opt
    {:include => {
      :comic => {
        :author => {}
      }
    }}
  end
  
  def allow? operators
    return nil if self.story_id == nil or self.comic_id == nil
    self.comic.own?(operators) and self.story.own?(operators)
  end
  
end
