class Sheet < Peta::Root
  load_manifest
  has_many :sheet_panels
  has_many :story_sheets
  belongs_to :author
  
  validates :caption, :presence => true, :length => {:maximum => 100}
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  accepts_nested_attributes_for :sheet_panels, :allow_destroy => true
  
  scope :find_index, -> do
    where(arel_table[:visible].gt 0)
  end
  
  scope :find_private, -> (operators) do 
    where(author_id: operators.author.id)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(author_id: author_id)
  end
  
  scope :with_panels, -> do
    includes(sheet_panels: :panel)
  end
  
  scope :find_by_panel, -> (panel_id) do 
    with_panels.find_index.where(Panel.arel_table[:id].eq panel_id).references(:panel)
  end
  
  scope :with_stories, -> do
    includes(story_sheets: :story)
  end
  
  scope :find_by_story, -> (story_id) do 
    with_stories.find_index.where(Story.arel_table[:id].eq story_id).references(:story)
  end
  
  # scope of find_play
  def self.find_play(id)
    SheetPanel.find_play(id)
  end
  
  # scope of find_private_play
  def self.find_private_play(id, operators)
    SheetPanel.find_private_play(id, operators)
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
    self.author_id = nil
  end
  
  def overwrite operators
    return false unless operators.author
    self.author_id = operators.author.id
  end
  
  def visible? operators
    case super
    when nil # super return
      return true
    when false
      return false
    else
      self.visible > 0
    end
  end
  
  def usable? operators
    self.visible? operators
  end
  
  def symbol_filename
  end
  
  def self.public_list_where list
    'sheets.visible > 0'
  end
  
  def self.show_opt
    {:include => {:sheet_panels => {:panel => {}}, :author => {}}}
  end
  
  def self.show_json_opt
    {:include => {:sheet_panels => {:include => {:panel => {}}}, :author => {}}}
  end
  
  def scenario
    panels.map {|panel|
      panel.scenario
    }.join
  end
  
  def plain_scenario
    panels.map {|panel|
      panel.plain_scenario
    }.join
  end
  
  def licensed_pictures
    r = {}
    self.scenario_elements.each do |elm|
      r.merge!(elm.licensed_pictures)
    end
    r
  end
  
  def copyable?
    r = true
    self.elements_items.each do |elm|
      next if elm.copyable?
      r = false
      break
    end
    r
  end
  
end
