#ストーリー
class Story < Peta::Binder
  load_manifest
  has_many :comic_stories
  has_many :story_sheets
  belongs_to :author
  
  validates :title, :presence => true, :length => {:maximum => 100}
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0..1}
  validates :author_id, :presence => true, :numericality => true, :existence => {:both => false}
  
  scope :find_index, -> do
    where(arel_table[:visible].gt 0)
  end
  
  scope :find_private, -> (operators) do 
    where(author_id: operators.author.id)
  end
  
  scope :find_by_author, -> (author_id) do 
    find_index.where(author_id: author_id)
  end
  
  scope :with_comics, -> do
    includes(comic_stories: :comic)
  end
  
  scope :find_by_comic, -> (comic_id) do 
    with_comics.find_index.where(Comic.arel_table[:id].eq comic_id).references(:comic)
  end
  
  scope :with_sheets, -> do
    includes(story_sheets: :sheet)
  end
  
  scope :find_by_sheet, -> (sheet_id) do 
    with_sheets.find_index.where(Sheet.arel_table[:id].eq sheet_id).references(:sheet)
  end
  
  # scope of find_play
  def self.find_play(id)
    StorySheet.find_play(id)
  end
  
  # scope of find_private_play
  def self.find_private_play(id, operators)
    StorySheet.find_private_play(id, operators)
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
    self.author_id = nil
  end
  
  def overwrite operators
    return false unless operators.author
    self.author_id = operators.author.id
    super()
  end
  
  def visible? operators
    case super
    when nil # super return
      return true
    when false
      return false
    else
      self.visible > 0
    end
  end
  
  def self.public_list_where list
    'stories.visible > 0'
  end
  
  def self.show_opt
    {:include => {:story_sheets => {:sheet => {}}, :author => {}}}
  end
  
end
