# -*- encoding: utf-8 -*-
#原画
require 'spec_helper'

describe OriginalPicture do
  include ManifestsMacros
  before do
    @model = OriginalPicture
  end
  
  describe 'ペタナイズに於いて' do
    it '絵師が投稿するコンテンツである' do
      expect(@model.ancestors.include?(Peta::Content)).to eq true
    end
    it '親モデルは持たない' do
      expect(@model.my_peta.parent_model_name).to be_blank
    end
    describe 'ブーストに於いて' do
      it '特筆すべき注意事項はない' do
        expect(@model.my_peta.boost.keys).to be_empty
      end
    end
    
  end
  
  describe 'コントローラに於いて' do
    before do
      @controller = @model.my_controller
    end
    it '原画のためのコントローラである' do
      expect(@model.item_name).to eq 'original_picture'
    end
    describe 'actionに於いて' do
      before do
        @actions = @controller.actions
        @actions_names = ['index', 'history', 'show', 
          'count', 
          'new', 'edit', 'create', 'update', 'destroy']
      end
      it 'これらのアクションが定義されている' do
        set_actions? @actions, @actions_names
      end
      describe 'baseに於いて' do
      end
      describe 'indexに於いて' do
        before do
          @action = @actions['index']
        end
        it 'listタイプのアクションである' do
          expect(@action.type).to eq 'list'
        end
        it 'マニフェストから設定を取り出している' do
          # オーナーしか操作できないアイテムなので、
          # 不特定多数のユーザで共有する必要がない。
          # よって、 privateリスト(自分の原画リスト)に割り当てる。 
          set_list_action? @action, 'original_picture', 'private'
        end
      end
      describe 'historyに於いて' do
        before do
          @action = @actions['history']
        end
        # 投稿履歴を一覧するための機能。
        # 一覧そのものは実素材の原画フィルタで見られるが、
        # 過去の実素材を他人が見る事はできないので、
        # オーナーであることが保証される原画にこの機能を備えてある。 
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'showに於いて' do
        before do
          @action = @actions['show']
        end
        it 'showタイプのアクションである' do
          expect(@action.type).to eq 'show'
        end
      end
      describe 'countに於いて' do
        before do
          @action = @actions['count']
        end
        it 'countタイプのアクションである' do
          expect(@action.type).to eq 'count'
        end
        it 'マニフェストから設定を取り出している' do
          # indexがprivateリストなので、当然件数取得もprivateリストの総数を返す。 
          set_list_action? @action, 'original_picture', 'private'
        end
      end
      describe 'newに於いて' do
        before do
          @action = @actions['new']
        end
        it 'newタイプのアクションである' do
          expect(@action.type).to eq 'new'
        end
        it 'type' do
          expect(@action.item_name).to eq 'original_picture'
        end
      end
      describe 'editに於いて' do
        before do
          @action = @actions['edit']
        end
        it 'editタイプのアクションである' do
          expect(@action.type).to eq 'edit'
        end
        it 'type' do
          expect(@action.item_name).to eq 'original_picture'
        end
      end
      describe 'createに於いて' do
        before do
          @action = @actions['create']
        end
        it 'createタイプのアクションである' do
          expect(@action.type).to eq 'create'
        end
        it 'type' do
          expect(@action.item_name).to eq 'original_picture'
        end
      end
      describe 'updateに於いて' do
        before do
          @action = @actions['update']
        end
        it 'updateタイプのアクションである' do
          expect(@action.type).to eq 'update'
        end
        it 'type' do
          expect(@action.item_name).to eq 'original_picture'
        end
      end
      describe 'destroyに於いて' do
        before do
          @action = @actions['destroy']
        end
        it 'destroyタイプのアクションである' do
          expect(@action.type).to eq 'destroy'
        end
        it 'type' do
          expect(@action.item_name).to eq 'original_picture'
        end
      end
    end
    
  end
  
  describe 'モデルに於いて' do
    before do
      @my_manifest = @model.my_manifest
    end
    it 'サポート機能が動作している' do
      expect(@my_manifest.model_name).to eq 'original_picture'
      expect(@my_manifest.classify).to eq @model
      expect(@my_manifest.table_name).to eq 'original_pictures'
    end
    
    describe 'associationsに於いて' do
      describe 'belongs_toに於いて' do
        before do
          @belongs_to = @my_manifest.associations.belongs_to
          @names = ['artist']
        end
        it 'これらのモデルに所属している' do
          set_associations? @belongs_to, @names
        end
        describe 'artistに於いて' do
          before do
            @mani = @belongs_to['artist']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名と外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'artist'
            expect(@mani.id_column).to eq 'artist_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq Artist
          end
        end
      end
      describe 'has_manyに於いて' do
        before do
          @has_many = @model.my_manifest.associations.has_many
          @names = ['pictures']
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_many, @names
        end
        describe 'picturesに於いて' do
          before do
            @mani = @has_many['pictures']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーと中間モデルが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'picture'
            expect(@mani.foreign_key).to eq 'original_picture_id'
            expect(@mani.through).to be nil
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている, 中間モデルフラグ
            expect(@mani.through?).to be false
            expect(@mani.model).to eq Picture
          end
        end
      end
      describe 'has_oneに於いて' do
        before do
          @has_one = @model.my_manifest.associations.has_one
          @names = ['resource_picture']
        end
        it 'これらのモデルを所持している' do
          set_associations? @has_one, @names
        end
        describe 'resource_pictureに於いて' do
          before do
            @mani = @has_one['resource_picture']
          end
          it 'マニフェストから設定を取り出している' do
            # モデル名, 外部キーが定義あるいはデフォルト値補充されている
            expect(@mani.model_name).to eq 'resource_picture'
            expect(@mani.foreign_key).to eq 'original_picture_id'
          end
          it 'サポート機能が動作している' do
            # モデルを再現できている
            expect(@mani.model).to eq ResourcePicture
          end
        end
      end
    end
    
    describe 'attributesに於いて' do
      before do
        @attributes = @model.my_manifest.attributes
      end
      it 'これらのカラムを定義している' do
        set_attributes? @attributes, 
          ['id', 'ext', 
          'width', 'height', 'filesize', 'artist_id', 'md5', 
          'uploaded_at', 'published_at', 'stopped_at', 
          'created_at', 'updated_at']
      end
      describe 'extに於いて' do
        before do
          @attribute = @attributes['ext']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'widthに於いて' do
        before do
          @attribute = @attributes['width']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'heightに於いて' do
        before do
          @attribute = @attributes['height']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'filesizeに於いて' do
        before do
          @attribute = @attributes['filesize']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'artist_idに於いて' do
        before do
          @attribute = @attributes['artist_id']
        end
        it 'numberタイプである' do
          expect(@attribute.type).to eq 'number'
        end
      end
      describe 'md5に於いて' do
        before do
          @attribute = @attributes['md5']
        end
        it 'textタイプである' do
          expect(@attribute.type).to eq 'text'
        end
      end
      describe 'uploaded_atに於いて' do
        before do
          @attribute = @attributes['uploaded_at']
        end
        it 'datetimeタイプである' do
          expect(@attribute.type).to eq 'datetime'
        end
      end
      describe 'published_atに於いて' do
        before do
          @attribute = @attributes['published_at']
        end
        it 'datetimeタイプである' do
          expect(@attribute.type).to eq 'datetime'
        end
      end
      describe 'stopped_atに於いて' do
        before do
          @attribute = @attributes['stopped_at']
        end
        it 'datetimeタイプである' do
          expect(@attribute.type).to eq 'datetime'
        end
      end
    end
    
  end
  
end
